$MGC::Version = 4; //1.1.1

if( isPackage( MinigolfClientPackage ) )
{
	deactivatePackage( MinigolfClientPackage );
}

package MinigolfClientPackage
{
	function resetCanvas()
	{
		Parent::resetCanvas();

		%this = nameToID( MinigolfHUD );

		if( !isObject( %this ) )
		{
			return;
		}

		%this.positionHUD();
	}

	function disconnectedCleanup( %a )
	{
		%return = Parent::disconnectedCleanup( %a );

		$GolfballDatablock = "";

		schedule( 1000, 0, clientCmdMGC_HUD_SetActive, false );
		schedule( 1000, 0, clientCmdMGC_Scores_ToggleGui, false );

		return %return;
	}
	function gameConnection::setConnectArgs(%client, %lan, %net, %prefix, %suffix, %password, %rtb, %arg7, %arg8, %arg9, %arg10, %arg11, %arg12, %arg13, %arg14, %arg15)
	{
		parent::setConnectArgs(%client, %lan, %net, %prefix, %suffix, %password, %rtb, %arg7, %arg8, %arg9, "Minigolf "@ $MGC::Version NL %arg10, %arg11, %arg12, %arg13, %arg14, %arg15);
	}
};

activatePackage( MinigolfClientPackage );

//function clientCmdMGC_Handshake()
//{
//	commandToServer( 'MGC_Handshake', $MGC::Version );
//	echo( "Minigolf Handshake (Client): Handshake offered. Response sent." );
//}

function MinigolfHUD::positionHUD( %this )
{
	if( %this.getGroup().getName() !$= "PlayGui" )
	{
		return;
	}

	%y = getWord( getRes(), 1 ) - getWord( %this.getExtent(), 1 );

	%this.tangoSetPosition( -150 SPC %y );
}

function clientCmdMGC_HUD_SetActive( %val )
{
	if( %val )
	{
		PlayGui.add( MinigolfHUD );
		MinigolfHUD.positionHUD();

		activatePackage("MinigolfCommandOverride");
	}
	else
	{
		%group = MinigolfHUD.getGroup();

		if(%group != -1 && %group.getName() $= "PlayGui" )
		{
			%group.remove( MinigolfHUD );
		}

		deactivatePackage("MinigolfCommandOverride");
	}
}

function clientCmdMGC_HUD_SetPower( %val )
{
	%val *= 1;

	%x = %val * 327;

	MGC_Powerbar.tangoSetPosition( "2 2" );
	MGC_Powerbar.tangoScaleTo( %x SPC 12, 100, sine );
}

function clientCmdMGC_HUD_SetLastPower( %val )
{
	%val *= 1;

	%x = %val * 327;

	MGC_LastStrokePower.tangoMoveTo( %x SPC 2, 500, elastic );
}

function clientCmdMGC_HUD_SetHoleData( %hole, %par )
{
	MGC_CurrHole.setText( %hole * 1 );
	MGC_CurrPar.setText( %par * 1 );
}

function clientCmdMGC_HUD_SetStrokeData( %stroke )
{
	MGC_CurrStroke.setText( %stroke * 1 );
}

function MinigolfScreenCanvas::setPixel( %this, %x, %y, %color )
{
	%color = mFloor( 255 - ( %this.c += 0.3 ) );

	if( %color < 0 ) %color = 0;

	%color = %color SPC %color SPC %color SPC 100;

	%pixel = new GuiBitmapCtrl()
	{
		position = %x SPC %y;
		extent = "10 10";
		minExtent = "1 1";
		clipToParent = "1";
		mColor = %color;
		bitmap = "Add-Ons/Client_Minigolf/res/img/px";
	};

	%this.add(%pixel);
}

function MinigolfScreenCanvas::drawLine( %this, %posa, %posb, %pixelSize )
{
	if( %pixelSize < 1 )
	{
		return false;
	}

	%this.pixelSize = mFloor(%pixelSize);
	%this.c=0;

	%vx0 = getWord(%posa, 0);
	%vy0 = getWord(%posa, 1);
	%vx1 = getWord(%posb, 0);
	%vy1 = getWord(%posb, 1);

	%x0 = %vx0;
	%x0 = mFloor(%x0 / %this.pixelSize) * %this.pixelSize;
	%x1 = %vx1;
	%x1 = mFloor(%x1 / %this.pixelSize) * %this.pixelSize;
	%y0 = %vy0;
	%y0 = mFloor(%y0 / %this.pixelSize) * %this.pixelSize;
	%y1 = %vy1;
	%y1 = mFloor(%y1 / %this.pixelSize) * %this.pixelSize;

	return %this.__drawLine( %x0, %y0, %x1, %y1 );
}

function MinigolfScreenCanvas::__drawLine( %this, %x0, %y0, %x1, %y1 )
{
	%dx = mAbs( %x1 - %x0 );
	%sx = %x0 < %x1 ? %this.pixelSize : -%this.pixelSize;
	%dy = mAbs( %y1 - %y0 );
	%sy = %y0 < %y1 ? %this.pixelSize : -%this.pixelSize; 
	%err = ( %dx > %dy ? %dx : -%dy ) / 2;

	while(true)
	{
		%this.setPixel( %x0, %y0 );
		if ( %x0 == %x1 && %y0 == %y1 ) { break; }
		%e2 = %err;
		if ( %e2 > -%dx ) { %err -= %dy; %x0 += %sx; }
		if ( %e2 < %dy ) { %err += %dx; %y0 += %sy; }
	}
}

function clientGolferTick()
{
	cancel( $clientGolferTick );

	if( !isObject( MinigolfScreenCanvas ) )
	{
		new GuiControl( MinigolfScreenCanvas )
		{
			horizSizing = "relative";
			vertSizing = "relative";
			position = "0 0";
			extent = PlayGui.extent;
		};
	
		PlayGui.add( MinigolfScreenCanvas );
	}

	%this = MinigolfScreenCanvas;

	//%screenCenter = getWords( VectorScale( getRes(), 0.5 ), 0, 1 );

	%ball = serverConnection.getControlObject().getOrbitObject();

	if( isObject( %ball ) )
	{
		// %screenCenter = %this.worldToScreen( %ball.getPosition() );
		%screenCenter = %this.worldToScreen($myLocation);
	}
	else
	{
		%this.deleteAll();
	}

	%mousePos = Canvas.getCursorPos();

	if( isObject( %ball ) && ( %screenCenter != %this.screenCenter || %mousePos != %this.mousePos ) )
	{
		%this.deleteAll();
		%this.drawLine( %mousePos, %screenCenter, 1 );

		%this.screenCenter = %screenCenter;
		%this.mousePos = %mousePos;

		%vs = VectorSub( %mousePos, %screenCenter );

		%yaw = getMouseAdjustAmount( getWord( %vs, 0 ) );
		%pitch = getMouseAdjustAmount( getWord( %vs, 1 ) );

		%vec = rotateVector( $myEyeVector, %yaw, %pitch );

		echo(%vec);

		%x = getWord( %vec, 0 );
		%y = getWord( %vec, 1 );

		%dx = getWord( %vs, 0 );
		%dy = getWord( %vs, 1 );

		//%x = -%x;

		%vec = %x SPC %y SPC 0;

		$vec = %vec;

		//echo($vec);
	}

	$clientGolferTick = schedule( 1, 0, clientGolferTick );
}

function MinigolfScreenCanvas::worldToScreen( %this, %xyz )
{
	//Full credits to whoever originally coded this ... Space Guy?

	%self = serverConnection.getControlObject();
	%posa = %this.getMyEyePoint();

	%fv = %self.getMuzzleVector(0); 
	%yp = %this.getYawPitch( %fv, %posa, %xyz );
	%yaw = getWord( %yp, 0 );
	%pitch = getWord( %yp, 1 );

	%halfx = getWord( $Pref::Video::Resolution, 0 ) / 2;
	%halfy = getWord( $Pref::Video::Resolution, 1 ) / 2;
	%fovx = mDegToRad( $CameraFov / 2 );
	%fovy = mATan (mTan( %fovx ) * %halfy / %halfx, 1 );

	if(%yaw > %fovx)
		%yaw = %fovx;
	if(%yaw < -%fovx)
		%yaw = -%fovx;
	if(%pitch > %fovy)
		%pitch = %fovy;
	if(%pitch < -%fovy)
		%pitch = -%fovy;

	%ratx = mTan(%yaw) * mCos(%fovx) / mSin(%fovx);
	%raty = mTan(%pitch) * mCos(%fovy) / mSin(%fovy);

	%x = mFloatLength( %ratx * %halfx + %halfx - getWord( %gui.extent, 0 ) / 2, 0 );
	%y = mFloatLength( %raty * %halfy + %halfy - getWord( %gui.extent, 1 ) / 2, 0 );

	%y -= ( %x - getWord( %gui.extent, 0 ) ) / 100;

	return %x SPC %y;
}

function MinigolfScreenCanvas::getYawPitch( %this, %fv, %posa, %posb )
{
	%x = getWord( %fv, 0 );
	%y = getWord( %fv, 1 );
	%vv = vectorNormalize( vectorSub( %posb, getWords( %posa, 0, 2 ) ) );
	%xx = getWord( %vv, 0 );
	%yy = getWord( %vv, 1 );
	
	%yaw = mATan( %xx, %yy ) - mATan( %x, %y );
	%pitch = mATan( getWord( %fv, 2 ), mSqrt( %x * %x + %y * %y ) ) - mATan( getWord( %vv, 2 ), mSqrt( %xx * %xx + %yy * %yy ) );
	
	if( %yaw > 3.14159 ) %yaw = -6.28318 + %yaw;
	if( %yaw < -3.14159 ) %yaw =	6.28318 + %yaw;
	if( %pitch > 3.14159 ) %pitch = -6.28318 + %pitch;
	if( %pitch < -3.14159 ) %pitch =	6.28318 + %pitch;
	
	return %yaw SPC %pitch;
}

function clientCmdReceiveMyEyePoint( %eyePoint, %eyeVector )
{
	$myEyePoint = %eyePoint;
	$myEyeVector = %eyeVector;

	//echo($Sim::Time);
}

function clientCmdReceiveMyLocation( %loc )
{
	$myLocation = %loc;
}

function MinigolfScreenCanvas::getMyEyePoint( %this )
{
	commandToServer( 'getMyEyePoint' );

	return $myEyePoint;
}

function MinigolfScreenCanvas::screenToWorld( %this, %xy )
{
	%cursorX = getWord(%xy, 0);
	%cursorY = getWord(%xy, 1);
	%resolution = getRes();
	%resolutionX = getWord(%resolution,0);
	%resolutionY = getWord(%resolution,1);
	%camerapos = serverConnection.getControlObject().getTransform();
	if(%camerapos $= "") return false;
	%cameraX = getWord(%camerapos, 0);
	%cameraY = getWord(%camerapos, 1);
	
	if(%cursorX < %resolutionX / 2)
	{
		%x = %cameraX - (%resolutionX / 2 - %cursorX) / 15;
	}
	else if(%cursorX > %resolutionX / 2)
	{
		%x = %cameraX + (%cursorX - %resolutionX / 2 ) / 15;
	}
	else
	{
		%x = %cameraX;
	}
	
	if(%cursorY < %resolutionY / 2)
	{
		%z = %cameraY - (%resolutionY / 2 - %cursorY) / 15;
	}
	else if(%cursorY> %resolutionX / 2)
	{
		%z = %cameraY + (%cursorY - %resolutionY / 2 ) / 15;
	}
	else
	{
		%z = %cameraY;
	}

	return %x SPC 0 SPC %z;
}

function vector2Drotate( %vec, %rot )
{
	%x = getWord( %vec, 0 );
	%y = getWord( %vec, 1 );

	 %xn = ( %x * mCos( %rot ) ) - ( %y * mSin( %rot ) );
	 %yn = ( %x * mSin( %rot ) ) + ( %y * mCos( %rot ) );

	 return %xn SPC %yn;
}

function vecToRad( %vec )
{
	%x = getWord( %vec, 0 );
	%y = getWord( %vec, 1 );

	%theta = mATan( %y / %x );
}

function vecToYawPitch( %vec )
{
	%pitch = mAcos( %z );
	%yaw = mAtan( %y, %x );

	return %yaw SPC %pitch;
}

function rotateVector( %vector, %yaw, %pitch )
{
	%angles = %pitch SPC 0 SPC %yaw;
	%matrix = matrixCreateFromEuler( %angles );

	return matrixMulVector( %matrix, %vector );
}

function MGC_Scores_Clear()
{
	%this = nameToID( MGC_ScoresMain );

	if( !isObject( %this ) )
	{
		return;
	}

	for( %i = 0; %i < %this.getCount(); %i++ )
	{
		%obj = %this.getObject( %i );

		if( %obj.isPlayer )
		{
			%obj.schedule( 5, delete );
		}
	}

	schedule( 10, 0, __MGC_Scores_PostClear );
}

function clientCmdMGC_Scores_Clear()
{
	MGC_Scores_Clear();
}

function __MGC_Scores_PostClear()
{
	%y = ( MGC_ScoresMain.getCount() * 30 ) + 2;

	MGC_ScoresMain.tangoScaleTo( 1036 SPC %y, 1000, elastic );
}

function MGC_Scores_AnimateFlyIn()
{
	MinigolfScoresGui.extent = PlayGui.getExtent();
	MinigolfScoresGui.position = PlayGui.getPosition();

	%start = MGC_ScoresMain.centerX();

	%x = getWord( %start, 0 );
	%y = getWord( getRes(), 1 );

	MGC_ScoresMain.tangoSetPosition( %x SPC %y );
	MGC_ScoresMain.tangoMoveTo( %x SPC 78, 600, expo );

	%start = MGC_ScoresTitle.centerX();

	%x = getWord( %start, 0 );

	MGC_ScoresTitle.tangoSetPosition( %x SPC -78 );
	MGC_ScoresTitle.tangoMoveTo( %x SPC 0, 600, expo );

	alxPlay( alxCreateSource( AudioGui, "Add-Ons/Client_Minigolf/res/snd/swoosh-in.wav" ) );
}

function MGC_Scores_AnimateFlyOut()
{
	%start = MGC_ScoresMain.centerX();

	%x = getWord( %start, 0 );
	%y = getWord( getRes(), 1 );

	MGC_ScoresMain.tangoSetPosition( %x SPC 78 );
	MGC_ScoresMain.tangoMoveTo( %x SPC %y, 500, expo );

	%start = MGC_ScoresTitle.centerX();

	%x = getWord( %start, 0 );

	MGC_ScoresTitle.tangoSetPosition( %x SPC 0 );
	MGC_ScoresTitle.tangoMoveTo( %x SPC -78, 500, expo );

	alxPlay( alxCreateSource( AudioGui, "Add-Ons/Client_Minigolf/res/snd/swoosh-out.wav" ) );
}

function clientCmdMGC_Scores_ToggleGui( %value , %override )
{
	cancel( $MGC::Schedule::ScoresHide );

	if( %value && ( %override || !$ScoreGUIOpen ) )
	{
		PlayGui.add( MinigolfScoresGui );

		schedule( 15, 0, MGC_Scores_AnimateFlyIn );

		$ScoreGUIOpen = 1;
	}
	else if ( !%value )
	{
		MGC_Scores_AnimateFlyOut();
		MGC_Scores_Clear();

		%group = MinigolfScoresGui.getGroup();

		if(%group != -1 && %group.getName() $= "PlayGui" )
		{
			$MGC::Schedule::ScoresHide = PlayGui.schedule( 500, remove, MinigolfScoresGui );
		}
		$ScoreGUIOpen = 0;
	}
}

function clientCmdMGC_Scores_SetPars( %s1, %s2, %s3, %s4, %s5, %s6, %s7, %s8, %s9, %s10, %s11, %s12, %s13, %s14, %s15, %s16, %s17, %s18 )
{
	for( %i = 1; %i < 19; %i++ )
	{
		%par = mFloor( %s[ %i ] );

		if( %i < 10 )
		{
			%front += %par;
		}
		else
		{
			%back += %par;
		}

		$MGC::Par[ %i ] = %par;

		%obj = nameToID( "MGCPar" @ %i );
		if( !isObject( %obj ) ) continue;

		%obj.setText( %par );
	}

	MGCParFront.setText( %front );
	MGCParBack.setText( %back );
	MGCParTotal.setText( %front + %back );
}

function clientCmdMGC_Scores_SetPars2( %hole, %par ) 
{
	%hole = %hole + 1;
	if ( %hole < 1 || %hole > 18 ) 
	{
		return;
	}

	%par = mFloor( %par );

	$MGC::Par[ %hole ] = %par;

	%obj = nameToID( "MGCPar" @ %hole );
	if( !isObject( %obj ) ) return;

	%obj.setText( %par );

	for ( %i = 1; %i < 19; %i++ ) 
	{
		if ( %i < 10 ) 
		{
			%front += $MGC::Par[ %i ];
		}
		else
		{
			%back += $MGC::Par[ %i ];
		}
	}

	MGCParFront.setText( %front );
	MGCParBack.setText( %back );
	MGCParTotal.setText( %front + %back );
}

function clientCmdMGC_Scores_AddPlayer( %name, %s )
{
	%name = strupr( %name );
	%name = %name !$= "" ? %name : "ERROR";

	%y = ( MGC_ScoresMain.getCount() * 30 ) + 2;

	for ( %i = 1; %i < 19; %i++ )
	{
		%s[ %i ] = getWord( %s, %i - 1 ) + 0;
	}

	%main = new GuiControl()
	{
		position = 2 SPC %y;
		extent = "1036 28";
		isPlayer = true;
	
		// Name
		new GuiSwatchCtrl()
		{
			position = "0 0";
			extent = "197 28";
			color = "57 109 36 190";
	
			new GuiTextCtrl()
			{
				profile = "MGCScore_BodyNameText";
				horizSizing = "center";
				vertSizing = "center";
				position = "3 0";
				extent = "190 28";
				text = %name;
			};
		};
	};

	%x = 199;

	for( %i = 1; %i < 19; %i++ )
	{
		%score = %s[ %i ];

		if( getSubStr( %score, 0, 1 ) $= "^" )
		{
			%scoreToSum = mFloor( getSubStr( %score, 1, strLen( %score ) - 1 ) );

			%score = "-";
		}

		else
		{
			%score = mFloor( %score );
			%scoreToSum = %score;
		}

		if( %i < 10 )
		{
			%front += %scoreToSum;
		}
		else
		{
			%back += %scoreToSum;
		}

		%this = new GuiSwatchCtrl()
		{
			position = %x SPC 0;
			extent = "30 28";
			color = "57 109 36 190";
		
			new GuiTextCtrl()
			{
				profile = ( %score > $MGC::Par[ %i ] ) ? "MGCScore_BodyRedText" : "MGCScore_BodyWhiteText";
				horizSizing = "center";
				vertSizing = "center";
				position = "0 0";
				extent = "30 28";
				text = %score ? %score : "-";
			};
		};
	
		%main.add( %this );

		%x += 32;

		if( %i == 9 )
		{
			%x += 80;
		}
	}

	%frontObj = new GuiSwatchCtrl()
	{
		position = "487 0";
		extent = "78 28";
		color = "57 109 36 190";
	
		new GuiTextCtrl()
		{
			profile = "MGCScore_BodyWhiteText";
			horizSizing = "center";
			vertSizing = "center";
			position = "0 0";
			extent = "78 28";
			text = %front ? %front : "-";
		};
	};

	%main.add( %frontObj );

	%backObj = new GuiSwatchCtrl()
	{
		position = "855 0";
		extent = "78 28";
		color = "57 109 36 190";
	
		new GuiTextCtrl()
		{
			profile = "MGCScore_BodyWhiteText";
			horizSizing = "center";
			vertSizing = "center";
			position = "0 0";
			extent = "78 28";
			text = %back ? %back : "-";
		};
	};

	%main.add( %backObj );

	%total = %front + %back;

	%totalObj = new GuiSwatchCtrl()
	{
		position = "935 0";
		extent = "97 28";
		color = "57 109 36 190";
	
		new GuiTextCtrl()
		{
			profile = "MGCScore_BodyWhiteText";
			horizSizing = "center";
			vertSizing = "center";
			position = "0 0";
			extent = "97 28";
			text = %total ? %total : "-";
		};
	};

	%main.add( %totalObj );

	MGC_ScoresMain.add( %main );

	MGC_ScoresMain.tangoScaleTo( 1036 SPC %y + 30, 1000, elastic );
}